"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useImportConnections = void 0;
const react_1 = require("react");
const provider_1 = require("@mongodb-js/connection-storage/provider");
const fs_1 = require("fs");
const common_1 = require("./common");
const provider_2 = require("@mongodb-js/compass-connections/provider");
const INITIAL_STATE = Object.freeze({
    ...common_1.COMMON_INITIAL_STATE,
    passphraseRequired: false,
    fileContents: '',
});
async function loadFile({ filename, passphrase, existingConnectionIds, }, deserializeConnections) {
    if (!filename) {
        return INITIAL_STATE;
    }
    try {
        const fileContents = await fs_1.promises.readFile(filename, 'utf8');
        const connectionList = [];
        const connections = await deserializeConnections({
            content: fileContents,
            options: {
                passphrase,
            },
        });
        for (const info of connections) {
            if (info.favorite?.name) {
                const isExistingConnection = existingConnectionIds.includes(info.id);
                connectionList.push({
                    name: info.favorite.name,
                    id: info.id,
                    selected: !isExistingConnection,
                    isExistingConnection,
                });
            }
        }
        if (connectionList.length === 0) {
            throw new Error('File does not contain any connections');
        }
        return {
            fileContents,
            connectionList,
            error: '',
        };
    }
    catch (err) {
        return {
            fileContents: '',
            connectionList: [],
            error: err.message,
            ...(err.passphraseRequired && { passphraseRequired: true }),
        };
    }
}
function useImportConnections({ finish, open, trackingProps, }) {
    const existingConnections = (0, provider_2.useConnectionsList)((conn) => {
        return !conn.isBeingCreated && !conn.isAutoconnectInfo;
    });
    const { importConnections } = (0, provider_2.useConnectionActions)();
    const connectionStorage = (0, provider_1.useConnectionStorageContext)();
    const deserializeConnectionsImpl = connectionStorage.deserializeConnections?.bind(connectionStorage);
    if (!deserializeConnectionsImpl) {
        throw new Error('Import Connections feature requires the provided ConnectionStorage to implement importConnections and deserializeConnections');
    }
    const [state, setState] = (0, react_1.useState)(INITIAL_STATE);
    (0, react_1.useEffect)(() => {
        // Reset the form state to initial when modal is open, but keep the list
        setState((prevState) => {
            return {
                ...INITIAL_STATE,
                connectionList: prevState.connectionList,
            };
        });
    }, [open]);
    const { passphrase, filename, fileContents, connectionList } = state;
    const existingConnectionIds = existingConnections.map(({ info }) => info.id);
    (0, react_1.useEffect)(() => {
        // If `existingConnections` changes, update the list of connections that are
        // displayed in our table.
        setState((prevState) => ({
            ...prevState,
            connectionList: state.connectionList.map((conn) => ({
                ...conn,
                isExistingConnection: existingConnectionIds.includes(conn.id),
            })),
        }));
    }, [existingConnectionIds.join(',')]);
    const { onChangeConnectionList, onChangePassphrase, onCancel } = (0, common_1.useImportExportConnectionsCommon)(setState, finish);
    const onSubmit = (0, react_1.useCallback)(() => {
        setState((prevState) => ({ ...prevState, inProgress: true }));
        void (async () => {
            const filterConnectionIds = connectionList
                .filter((x) => x.selected)
                .map((x) => x.id);
            try {
                await importConnections({
                    content: fileContents,
                    options: {
                        passphrase,
                        filterConnectionIds,
                        trackingProps: {
                            ...trackingProps,
                            connection_ids: filterConnectionIds,
                        },
                    },
                });
            }
            catch (err) {
                setState((prevState) => {
                    return {
                        ...prevState,
                        inProgress: false,
                        error: err.message,
                    };
                });
                return;
            }
            finish('succeeded');
        })();
    }, [connectionList, fileContents, passphrase, finish]);
    const LOAD_CONNECTIONS_FILE_DEBOUNCE_DELAY = 100;
    (0, react_1.useEffect)(() => {
        let timer;
        timer = setTimeout(() => {
            timer = undefined;
            void loadFile({ filename, passphrase, existingConnectionIds }, deserializeConnectionsImpl).then((stateUpdate) => {
                setState((prevState) => {
                    if (
                    // Only update the state if filename and passphrase haven't changed
                    // while loading the connections list
                    filename === prevState.filename &&
                        passphrase === prevState.passphrase)
                        return { ...prevState, ...stateUpdate };
                    return prevState;
                });
            });
        }, LOAD_CONNECTIONS_FILE_DEBOUNCE_DELAY);
        return () => {
            if (timer !== undefined)
                clearTimeout(timer);
        };
    }, [filename, passphrase, existingConnectionIds.join(',')]);
    const onChangeFilename = (0, react_1.useCallback)((filename) => {
        setState((prevState) => ({
            ...prevState,
            filename,
            ...(filename !== prevState.filename && {
                error: '',
                passphraseRequired: false,
            }),
        }));
    }, []);
    return {
        onCancel,
        onSubmit,
        onChangeFilename,
        onChangePassphrase,
        onChangeConnectionList,
        state,
    };
}
exports.useImportConnections = useImportConnections;
//# sourceMappingURL=use-import-connections.js.map